# welcome to saturnOS config, run nixos-help if you need it .
{ config, pkgs, lib, ...  }:
let 
	projectRoot = builtins.toString ./.;
	unstableTarball = builtins.fetchTarball 
		"https://github.com/NixOS/nixpkgs/archive/nixos-unstable.tar.gz";
	homeManagerTarball = builtins.fetchTarball 
		"https://github.com/nix-community/home-manager/archive/master.tar.gz";
	homeManager = import "${homeManagerTarball}/nixos";
	deviceConf = import "${projectRoot}/hosts/${sysConf.host}/";
	userDir = "${projectRoot}/users/${sysConf.user}/";
	userConf = import userDir;
	unstable = import unstableTarball config.nixpkgs.config;
	sysConf = import ./saturn/system.nix;
in {
	imports = [	
		homeManager
		deviceConf
		userConf
		./modules
	];

	# import system.nix configuration
	saturn.hostname = sysConf.host;
	saturn.prettyHostname = sysConf.prettyHost;

	# nix options
	nix.settings = {
		experimental-features = "nix-command flakes";
		auto-optimise-store = true;
	};

	# cleanup packages older than 30 days from nix store
	nix.gc = {
		automatic = true;
		dates = "weekly";
		options = "--delete-older-than 30d";
	};

	# bootloader
	boot.loader.systemd-boot.enable = true;
	boot.loader.efi.canTouchEfiVariables = true;
	boot.loader.efi.efiSysMountPoint = "/boot/efi";
	boot.initrd.systemd.enable = true;
	boot.plymouth.enable = true;
	boot.kernelParams = ["quiet"];

	# configure networking
	networking.networkmanager.enable = true;
	networking.hosts = {
		"162.255.119.254" = ["www.librepunk.club"];
		"129.21.49.69" = ["www.librepunk.club"];
	};

	# configure locale
	time.timeZone = "Etc/UTC";
	i18n.defaultLocale = "en_CA.UTF-8";

	# enable gnome
	services.xserver.enable = true;
	services.xserver.displayManager.gdm.enable = true;
	services.xserver.desktopManager.gnome.enable = true;

	# configure x11 keymap
	services.xserver = {
		layout = "us";
		xkbVariant = "";
	};

	# enable dconf configuration for gnome and other supported applications
	programs.dconf.enable = true;

	# configure pipewire
	sound.enable = true;
	hardware.pulseaudio.enable = false;
	security.rtkit.enable = true;
	services.pipewire = {
		enable = true;
		alsa.enable = true;
		alsa.support32Bit = true;
		pulse.enable = true;
		jack.enable = true;
	};

	# add and configure users
	users.users.${sysConf.user} = {
		initialPassword = "";
		isNormalUser = true;
		description = "${sysConf.user}";
		extraGroups = [ "networkmanager" "wheel" ];
	};

	# import home-manager config from home.nix
	home-manager.users.${sysConf.user} = import "${userDir}/home.nix" sysConf;

	# install packages
	# `nix search` to add more
	nixpkgs.config.allowUnfree = true;
	environment.systemPackages = with pkgs; [
		pass
		git
		neovim
	];

	# set fish as default shell for all users
	users.defaultUserShell = unstable.fish;

	# suid wrapper ( for things which need privelidged acccess, or additional configuration )
	programs.mtr.enable = true;
	programs.gnupg.agent = {
		enable = true;
		enableSSHSupport = true;
		pinentryFlavor = "gnome3";
	};
	programs.fish.enable = true;

	# enable configure services
	services.printing.enable = true;	# printing
	services.openssh.enable = true;	# remote shell
	services.flatpak.enable = true;	#flatpak lol

	system.stateVersion = "22.11";
}
